package dao

import (
	"fmt"
	dbModel "sdk_api/model/db"
	"sync"
	"time"

	"gorm.io/gorm"
	"gorm.io/gorm/clause"
)

func (d *Dao) CreateUser(user *dbModel.User) (err error) {
	return d.db.Clauses(clause.OnConflict{
		Columns: []clause.Column{{Name: "user_id"}, {Name: "chat_id"}},
		DoUpdates: clause.Assignments(map[string]interface{}{
			"left_at":    gorm.Expr("NULL"),
			"updated_at": gorm.Expr("NOW()"),
		}),
	}).Create(user).Error
}

func (d *Dao) IncrMessageCount(a *dbModel.Active) (err error) {
	a.MsgCount = 1
	return d.db.Clauses(clause.OnConflict{
		Columns: []clause.Column{{Name: "user_id"}, {Name: "chat_id"}, {Name: "unix_day"}},
		DoUpdates: clause.Assignments(map[string]interface{}{
			"msg_count":  gorm.Expr(fmt.Sprintf("%s.msg_count + ?", a.TableName()), 1),
			"updated_at": gorm.Expr("NOW()"),
		}),
	}).Create(a).Error
}

func (d *Dao) GetUserActiveMsgCount(userId, chatId, unixDay int) (count int, err error) {
	a := dbModel.Active{}
	err = d.db.Model(&dbModel.Active{}).
		Select("msg_count").
		Where("user_id = ? AND chat_id = ? AND unix_day = ?", userId, chatId, unixDay).
		First(&a).Error

	if err == gorm.ErrRecordNotFound {
		return 0, nil
	}

	return a.MsgCount, err
}

func (d *Dao) UserExist(userId, chatId int) (dbUser *dbModel.User, err error) {
	dbUser = &dbModel.User{}
	err = d.db.Model(&dbModel.User{}).
		Select([]string{"username", "first_name", "last_name"}).
		Where("user_id = ? AND chat_id = ? AND left_at IS NULL", userId, chatId).
		First(&dbUser).Error
	if err == gorm.ErrRecordNotFound {
		return nil, nil
	}
	return dbUser, err
}

func (d *Dao) UpdateUserName(userId, chatId int, username, firstName, lastName string) (err error) {
	return d.db.Model(&dbModel.User{}).
		Where("user_id = ? AND chat_id = ? AND first_name = ? AND last_name = ? AND left_at IS NULL", userId, chatId, firstName, lastName).
		Update("username", username).Error
}

func (d *Dao) UserLeft(userId, chatId int) (err error) {
	return d.db.Model(&dbModel.User{}).
		Where("user_id = ? AND chat_id = ? AND left_at IS NULL", userId, chatId).
		Update("left_at", gorm.Expr("NOW()")).Error
}

var (
	supportedChatsCache     = map[int]time.Time{}
	supportedChatsCacheLock = sync.RWMutex{}
)

func (d *Dao) IsSupportedChat(chatId int) (exist bool, err error) {
	supportedChatsCacheLock.RLock()
	expired, ok := supportedChatsCache[chatId]
	supportedChatsCacheLock.RUnlock()
	if ok && expired.After(time.Now()) {
		return true, nil
	}

	a := dbModel.ChatGroup{}
	err = d.db.Model(&dbModel.ChatGroup{}).
		Where("chat_id = ?", chatId).
		First(&a).Error
	if err == gorm.ErrRecordNotFound {
		return false, nil
	}

	if err != nil {
		return false, err
	}

	supportedChatsCacheLock.Lock()
	supportedChatsCache[chatId] = time.Now().Add(1 * time.Hour)
	supportedChatsCacheLock.Unlock()
	return true, err
}

func (d *Dao) addTestGroup() (err error) {
	return d.db.Clauses(clause.OnConflict{DoNothing: true}).Create([]dbModel.ChatGroup{
		{
			ChatId:    -1002174503961,
			ChatTitle: "mail_notice",
		},
	}).Error
}
