package messager

import (
	"sdk_api/dao"
	dbModel "sdk_api/model/db"

	tgbotapi "github.com/go-telegram-bot-api/telegram-bot-api/v5"
	log "github.com/sirupsen/logrus"
)

type Messenger struct {
	botToken string
	bot      *tgbotapi.BotAPI
	d        *dao.Dao
}

func NewMessenger(botToken string, _d *dao.Dao) *Messenger {
	bot, err := tgbotapi.NewBotAPI(botToken)
	if err != nil {
		panic(err)
	}

	log.WithField("account", bot.Self.UserName).Info("Authorized on account")
	return &Messenger{botToken: botToken, bot: bot, d: _d}
}

func (m *Messenger) Start() {
	u := tgbotapi.NewUpdate(0)
	u.Timeout = 60

	updates := m.bot.GetUpdatesChan(u)
	for update := range updates {
		if update.Message == nil ||
			update.Message.IsCommand() {
			continue
		}

		supported, err := m.d.IsSupportedChat(int(update.Message.Chat.ID))
		if err != nil {
			log.WithError(err).Error("check supported chat error")
			continue
		}
		if !supported {
			log.WithField("chat_id", update.Message.Chat.ID).Debug("not supported chat")
			continue
		}
		var caught bool
		caught = m.handleUserLeft(update.Message)
		if caught {
			continue
		}
		caught = m.handleNewUser(update.Message)
		if caught {
			continue
		}
		m.handleActive(update.Message)
	}
}

func (m *Messenger) ExistGroupChat(userId, chatId int64) (exist bool, err error) {
	mem, err := m.bot.GetChatMember(tgbotapi.GetChatMemberConfig{
		ChatConfigWithUser: tgbotapi.ChatConfigWithUser{
			ChatID: chatId,
			UserID: userId,
		},
	})

	if err != nil {
		return
	}

	switch {
	case mem.HasLeft(), mem.WasKicked():
		return false, nil
	default:
		return true, nil
	}
}

func (m *Messenger) GetChatInfo(chatId int) (title, name string, err error) {
	chat, err := m.bot.GetChat(tgbotapi.ChatInfoConfig{ChatConfig: tgbotapi.ChatConfig{ChatID: int64(chatId)}})
	if err != nil {
		log.WithError(err).Error("get chat info error")
		return

	}
	return chat.Title, chat.UserName, nil
}

func (m *Messenger) handleNewUser(msg *tgbotapi.Message) (caught bool) {
	for _, user := range msg.NewChatMembers {
		log.WithFields(log.Fields{
			"user_id":    user.ID,
			"username":   user.UserName,
			"first_name": user.FirstName,
			"last_name":  user.LastName,
			"chat_id":    msg.Chat.ID,
			"chat_title": msg.Chat.Title,
			"is_bot":     user.IsBot,
		}).Debug("new user")

		if user.IsBot {
			continue
		}

		user := &dbModel.User{
			UserId:    int(user.ID),
			ChatId:    int(msg.Chat.ID),
			Username:  user.UserName,
			FirstName: user.FirstName,
			LastName:  user.LastName,
		}
		if err := m.d.CreateUser(user); err != nil {
			log.WithError(err).Error("create user error")
		}
		if !caught {
			caught = true
		}
	}
	return caught
}

func (m *Messenger) handleUserLeft(msg *tgbotapi.Message) (caught bool) {
	if msg.LeftChatMember == nil {
		return
	}
	log.WithFields(log.Fields{
		"user_id":    msg.LeftChatMember.ID,
		"username":   msg.LeftChatMember.UserName,
		"first_name": msg.LeftChatMember.FirstName,
		"last_name":  msg.LeftChatMember.LastName,
		"chat_id":    msg.Chat.ID,
		"chat_title": msg.Chat.Title,
		"is_bot":     msg.LeftChatMember.IsBot,
	}).Debug("user left")

	user := msg.LeftChatMember

	if user.IsBot {
		return
	}

	err := m.d.UserLeft(int(user.ID), int(msg.Chat.ID))
	if err != nil {
		log.WithError(err).Error("user left error")
	}
	return true
}

func (m *Messenger) handleActive(msg *tgbotapi.Message) {
	text := msg.Text
	if len(text) > 30 {
		text = text[:30] + "..."
	}
	log.WithFields(log.Fields{
		"user_id":    msg.From.ID,
		"username":   msg.From.UserName,
		"first_name": msg.From.FirstName,
		"last_name":  msg.From.LastName,
		"chat_id":    msg.Chat.ID,
		"chat_title": msg.Chat.Title,
		"is_bot":     msg.From.IsBot,
		"text":       text,
	}).Debug("new msg")

	if msg.From.IsBot {
		return
	}

	dbUser, err := m.d.UserExist(int(msg.From.ID), int(msg.Chat.ID))
	if err != nil {
		log.WithError(err).Error("exist user error")
		return
	}
	if err == nil && dbUser == nil {
		err = m.d.CreateUser(&dbModel.User{
			UserId:    int(msg.From.ID),
			ChatId:    int(msg.Chat.ID),
			Username:  msg.From.UserName,
			FirstName: msg.From.FirstName,
			LastName:  msg.From.LastName,
		})
		if err != nil {
			log.WithError(err).Error("create user 2 error")
			return
		}
	}

	if dbUser != nil && (dbUser.Username != msg.From.UserName || dbUser.FirstName != msg.From.FirstName || dbUser.LastName != msg.From.LastName) {
		err = m.d.UpdateUserName(int(msg.From.ID), int(msg.Chat.ID), msg.From.UserName, msg.From.FirstName, msg.From.LastName)
		if err != nil {
			log.WithError(err).Error("update user name error")
			return
		}
		log.WithFields(log.Fields{
			"user_id":       msg.From.ID,
			"chat_id":       msg.Chat.ID,
			"old_username":  dbUser.Username,
			"old_firstname": dbUser.FirstName,
			"old_lastname":  dbUser.LastName,
			"new_username":  msg.From.UserName,
			"new_firstname": msg.From.FirstName,
			"new_lastname":  msg.From.LastName,
		}).Debug("update user name")
	}

	err = m.d.IncrMessageCount(&dbModel.Active{
		UserId:  int(msg.From.ID),
		ChatId:  int(msg.Chat.ID),
		UnixDay: msg.Date / 86400,
	})
	if err != nil {
		log.WithError(err).Error("incr message count error")
	}
}

func (m *Messenger) SendMessage(chatId int64, msg string) error {
	_, err := m.bot.Send(tgbotapi.NewMessage(chatId, msg))
	return err
}
